
#ifndef __INFILE_MDL_HPP__
#define __INFILE_MDL_HPP__
/** Input Device (file) class library.
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (水, 03  7月 2013) $
 * $Id: InFile_mdl.hpp 56 2013-07-03 11:40:14Z vlbi $ 
 */


#include "Thread.h"
#include "InDev_mdl.hpp"
#include "tds_file.hpp"

// const int  DEFAULT_NPAGE = 4;
const long DEFAULT_NANO_SLEEP = 1000;

/** 
 * Input model for file reading. Succeeding from InDev_Model.
 */

class InFile_Model:public InDev_Model{
public:
  InFile_Model(); ///< Constructor
  ~InFile_Model(); ///< Destructor 
  /// Mapping data page of the epoch to reference of vector array
  const IndevReturns_t 
  MappingData(vector <uint8_t>& pntr, const time_t epoch=0);
  /// Mapping data page, pinter verison
  const uint8_t* MappingData(time_t epoch); 
  /// Release mapped page
  const IndevReturns_t UnMapping(const time_t data_epoch);
  /**  Returns page_size(1sec) in 4-byte unit  */
  uint32_t PageLength(){  return dpage.Pagesize()/4;} // (byte)
  const int OpenDev(const DevCheckLevel_t dclevel=STRICT); ///< Open device/file 
  const int CloseDev(); ///< Open device/file 
  /** Test device/file Permission and more 
   * @return 
   *  - -1: Faile to open
   *  -  0: File Open success  
   *  -  1: Success to get DAS info
   */
  const int TestDev(); 
  /// Returns Device type in string 
  string StrDevType(){string ret = "FILE"; return ret;}
  /// Returns Device type 
  InputModelType_t DevType(){return InDev_Model::FILE;}
  //  void Set_npage(const int n){dpage.Set_Npage(n);}
  void *thread_func();///< Thread function to get data
  /// Return page number of the earliest epoch
  int FirstPage();
  int InitPage(); ///< Initializing Page size with DAS_info
  /// Returns Page sate in string 
  string  StrPageState(const int lvl=0){return dpage.StrPageState(lvl);}
  /// Returns Epoch of the i page;  
  time_t Epoch(const int i)const{ return dpage.Epoch(i);}
  /// Set Das info
  int Set_DasInfo(const DAS_Info& inf); 
  void PageEnd(){dpage.PageEnd();} ///< Set DATA_ENbD state  to all pages
  /// Set input filename 
  int Set_infile(const char * fname);
  void Stop(){InDev_Model::Stop(); CloseDev();}
  /** function to return number of pages.   */
  const int  Npage(){return dpage.Npage();}
private:
  int ReadingData_vssp(); ///< Reading data to Buffer in background
  int ReadingData_mk5b(); ///< Reading data to Buffer in background
  int ReadingData_vdif(); ///< Reading data to Buffer in background
  int ReadingData_vsi();  ///< Reading data to Buffer in background
  string infile ; ///< input file name
  DataPage dpage; 
  TDS_FILE *tds_file; 
  InFile_Model(InFile_Model& mdl){;} /// < Copy Constructor, prohibited
    //  struct timespec t_wait;
};

inline  int InFile_Model::FirstPage() {
  time_t t1 = dpage.EarliestEpoch();
  cerr << "FirstPage(): t1 = " << t1 << endl;
  if( t1 == 0){
    if( Status() == RUNNING)  return -1; // Expecting recover
    else {
      cerr << "E InFile_mdl::FirstPage(): First page is 0 and thread is not running.\n";
      return -2;  // Hopless to recover, fatal.
    }
  }  
  return (t1 % dpage.Npage());
}


#endif
