/** 
 * @Author M.Sekido
 * @Version $Rev: 179 $
 * @Date  $Date: 2016-01-08 20:15:07 +0900 (金, 08  1月 2016) $
 * $Id: InVSI-spctr.cpp 179 2016-01-08 11:15:07Z vlbi $ 
 */
#include "InVSI_mdl.hpp"
#include "bitutl.hpp"
#include "pltutl.hpp"
#include "ExtSubCh.hpp"
#include "fftw_utl.hpp"


char* me;

const char* version =  "$Rev: 179 $ Updated $Date: 2016-01-08 20:15:07 +0900 (金, 08  1月 2016) $ by $Author: vlbi $";

void LogSetup(const int verbose);

void usage(){
  cerr << "Usage: " << me << "[-h] [-n <fft npt(x 1024 point)> ] [-b <bit>]"   << endl; 
  cerr << "  [-d <bit>]  [-c <ich>] [-a <npt Average>] [-e <n>] [-1 <n>] [-p <n>]. \n";
  cerr << " Program to display spectrum. \n";
  cerr << " -b : Quantization bit (default=2) \n";
  cerr << " -d : decode bit (default=2).\n";
  cerr << " -n : Number of FFT points. (default 2 x 1024) \n";
  cerr << " -a : Number of Average points. (default 100) \n";
  cerr << " -c : Channel Number. (default 0) \n";
  cerr << " -e : 2bit Encoding mode. (default 0: BINARY) \n";
  cerr << "     Choices  are 0:   Binary,00,01,10,11\n";
  cerr << "                  1:RevBinary,00,10,01,11\n";
  cerr << "                  2:SignMag  ,01,00,10,11\n";
  cerr << "                  3:RevSigMag,10,00,01,11\n";
  cerr << " -1 : 1bit Extracting from 2bit data. (default 0:MSB) \n";
  cerr << "      Choices are 0: MSB, or 1:LSB \n";
  cerr << " -p : Number(1,2,4,8,16) of Panel division for multiple output.(default 4)\n";
  cerr << " -h : Help \n";
  //  cerr << " -v : Verbose Level 0-4.\n";
  cerr << " " << version << std::endl;
  exit(1); 

}

int main(int argc, char* argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  int verbose_level =0;
  int k=0;
  //  int navg_k = 10;
  int navg=100;
  int qbit=2;
  int dbit =2;
  long nfft = 2 * 1024;
  int ich=0;
  int ndisplay = 4;
  int Enc_mode(0);
  Decode::BitEnc_t enc_type = Decode::BINARY_2bit;
  ExtSubCh::BitAsgn_t bit_asign = ExtSubCh::MSB1of2;
  string str1bit_asgn = "1bit:MSB";
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hn:v:b:d:a:c:1:e:p:")) { 
    case 'v':  verbose_level = atoi(optarg);   break;
    case 'n':   nfft = atoi(optarg) * 1024;   break;
    case 'a':   navg = atoi(optarg);   break;
    case 'b':   qbit = atoi(optarg);   break;
    case 'd':   dbit = atoi(optarg);   break;
    case 'c':   ich = atoi(optarg);   break;
    case 'p':   ndisplay = atoi(optarg);   break;
    case 'e':   Enc_mode = atoi(optarg);  break;
    case '1':   
      if(atoi(optarg) !=0){
	bit_asign =ExtSubCh::LSB1of2; str1bit_asgn = "1bit:LSB";  } break;
    case 'h':   
    case '?':    usage(); break;
    }
  }

  if(dbit > qbit){
    fprintf(stderr, "E: Output bit(%d) > input bit(%d), impossible\n", dbit,qbit);
    exit(1);
  }
  string str2bit_enc;
  if( dbit == 2){
    switch(Enc_mode){
    case 0: enc_type = Decode::BINARY_2bit; str2bit_enc = "BINARY_2bit"; break;
    case 1: enc_type = Decode::RevBINARY_2bit; str2bit_enc = "RevBINARY_2bit";break;
    case 2: enc_type = Decode::SignMag_2bit; str2bit_enc = "SignMag_2bit";break;
    case 3: enc_type = Decode::RevSgnMag_2bit; str2bit_enc = "RevSgnMag_2bit";break;
    default: fprintf(stderr, "E: 2bit-Encode mode wrong(%d)\n", Enc_mode);
      usage(); break;
    }
  }

  //  ulong navg = navg_k * 1000 ;
  string Stname("Kashima34");
  string Stid("kb");

  int nch = 32/qbit;
  DAS_Info indas( 64000000, nch, qbit );
  DAS_Info outdas( 64000000, nch, dbit );
  LogSetup(verbose_level);
   
  K5_VSI *k5vsi = new K5_VSI;
  InDev_Model *input_mdl = k5vsi;
  VLBI_Frame* in_vlbi_frame = new K5VSI_Frame;
  in_vlbi_frame->Set_DasInfo( indas );

  input_mdl->Set_VlbiFrame(in_vlbi_frame);
  input_mdl->Set_StnameID(Stname.c_str(),Stid.c_str());

  if( input_mdl->OpenDev(InDev_Model::MONIT) < 0){
    cerr << "E K5_VSI::Ready() failed\n";
    exit(1);
  };
  //  input_mdl->start(); // reading data Started 
  
  uint32_t pagesize = input_mdl->PageLength();
  int npage = input_mdl->Npage();
  printf("Number of Page = %d pages\n", npage );
  printf("Page-size = %u MByte\n", pagesize*4/1000000 );
  int first_page = input_mdl->FirstPage();
  printf("PgStat=[%s]\n", input_mdl->StrPageState(1).c_str());
  
  if( (uint32_t)(nfft*navg) > pagesize){
    fprintf(stderr,"E: Nfft(%ld)*navg(%d)=%lu > pagesize(%lu)\n", 
	    nfft, navg, nfft*navg, (ulong)pagesize);
    exit(1);
  }

  int ipage = (first_page + npage-1) % npage;
  time_t page_epoch = k5vsi->GetFrameEpoch(ipage);
  struct tm tm1;
  char dumbuf[256];
  gmtime_r ( &page_epoch, &tm1 );
  strftime ( dumbuf,256,"%Y/%j-%H:%M:%S",&tm1 );
  printf("FirstPage = %d ipage=%d, Epoch=%s\n", 
	 first_page, ipage, dumbuf);

  //  time_t t1;
  //  uint32_t ClockMHz = input_mdl->PageLength()/1000000;
  const uint8_t* mapped_page(0);
  
  Bit_hist b_hist(qbit);
  double dt0 = GetTime();
  Plt_inf plt_info;
  ExtSubCh extch;
  Fftwf fftw(nfft);
  ulong ext_byte = nfft*navg*4/qbit*dbit;
  vector <uint8_t> arrout(ext_byte);
  Decode decoder(&outdas);

  decoder.Make_Table(enc_type);

  vector <FComplex> carr(nfft);
  vector<float> amparr(nfft);
  vector<float> avg_amparr(nfft/2);
  vector<float> xarr(nfft/2);
  for(int i=0;  i<(int)xarr.size(); i++)  xarr[i] =(float)i;
  vector<float> phsarr(nfft);
  cpgopen("/xw");
  plt_info.Set_label("Freq (arb.u.)","Power (arb.u.)","Spectrum");
  plt_info.Add_opt("","L");
  cpgsvp(0.15,0.9,0.15,0.9);
  mult_x<float> mult;
  mult.Set_cff(1./logf(10));
  switch(ndisplay){
  case 1: cpgsubp(1,1); break;
  case 2: cpgsubp(1,2); break;
  case 8: cpgsubp(4,2); break;
  case 16: cpgsubp(4,4); break;
  case 4: 
  default: cpgsubp(2,2);  break;
  }
  
  cpgask(0);
  while(1){
    mapped_page = k5vsi->MappingData(page_epoch);    
    if(mapped_page == 0){
      gmtime_r ( &page_epoch, &tm1 );
      strftime ( dumbuf,256,"%Y/%j-%H:%M:%S",&tm1 );
      fprintf(stderr, "E Failed in mapping (%s)\n", dumbuf);
    break;
    }
    extch.Setup(mapped_page, ext_byte, &indas, outdas, bit_asign );
    if(dbit == 1)
      extch.Extract1bit_from2bit(arrout, arrout.size());
    else{
      extch.Extract(arrout,arrout.size());
    }
    decoder.Interpret(arrout, nfft*navg);

    ///***** Drawing Graphics ******* */
    cpgbbuf();
    for(int jch =0; jch<ndisplay && ich+jch < nch; jch++){
      const float *pf = decoder.pfValArr(ich+jch);
      bzero(&avg_amparr[0],sizeof(float)*avg_amparr.size());
      for(int iavg=0; iavg< navg; iavg++){
	fftw.Set_in(pf);
	fftw.Exec(-1);
	fftw.Shift_out_1(&carr[0]);
	FCarr2AParr(carr,amparr,phsarr);
	for(int i=0; i< (int)avg_amparr.size(); i++)
	  avg_amparr[i] = avg_amparr[i]*iavg/(iavg+1) 
	    + amparr[i]/(iavg+1);
	pf+= nfft;
      }
      
      transform(avg_amparr.begin(), avg_amparr.end(),
		avg_amparr.begin(), Log<float>());
      transform(avg_amparr.begin(), avg_amparr.end(),
		avg_amparr.begin(), mult);
      cpgpage();      
      plt_info.Plot1(xarr,avg_amparr);
      {
	cpgsave();
	char buf[512];
	sprintf(buf,"CH=%d ", ich+jch);
	string msg = buf;
	if(dbit==2) msg += str2bit_enc;
	if(dbit==1 && qbit ==2) msg += str1bit_asgn;
	cpgswin(0.,1.0, 0., 1.);
	cpgptxt(0.1, 0.9,0.,0., msg.c_str());
	cpgunsa();
      }
    }// End of for()
    cpgebuf();

    //************* Sleeping Time Keeper ********************
    dt0 += 1.0;
    double dt1 = dt0 - GetTime();
    if(dt1 > 0.){
      //     fprintf(stderr," %.2fl sec sleep.", dt1);
      unsigned long usec = (ulong)(dt1*1000000);
      usleep(usec);
    }
    page_epoch++;
    //    fprintf(stderr,"..next page..\n "); 
 }
  cpgclos();

}


void LogSetup(const int verbose=-1){

  string logdir = "/tmp/";
  string error_file = logdir + me +"_error.txt";
   
  string debug_file = logdir + me + "_dbg.txt";
  Logsystem::debug_logger()->set_stream(debug_file);
  string info_file = logdir + me + "_info.txt";
  Logsystem::info_logger()->set_stream(info_file);
  string warn_file = logdir + me + "_warn.txt";
  Logsystem::warning_logger()->set_stream(warn_file);
  string err_file = logdir + me + "_err.txt";
  Logsystem::error_logger()->set_stream(err_file);
      
  switch(verbose){
  case -1:
    cerr << " No Logging." << endl;
    break;
  case 0:
    Logsystem::debug_logger()->enable_logging(Logger::MINIMUM); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->set_stream(error_file);
    Logsystem::error_logger()->enable_logging();
    break;
  case 1:
    Logsystem::debug_logger()->enable_logging(Logger::SMALL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 2:
    Logsystem::debug_logger()->enable_logging(Logger::MEDIUM); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 3:
    Logsystem::debug_logger()->enable_logging(Logger::DETAIL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 4:
  default:
    Logsystem::debug_logger()->enable_logging(Logger::ALL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  }
  
}
